//
//================================================================================
//       Freescale Layerscape(TM) LS1021A Platform Software
//       (c) Copyright DAB-Embedded. All Rights Reserved.
//
// Use of this software is controlled by the terms and conditions found
// in the license agreement under which this software has been supplied.
//
//================================================================================
//
//
//  File:  sdk_gpio.h
//
//  This header defines interface for GPIO device driver. This driver control
//  GPIO pins on hardware. It allows abstract GPIO interface and break up
//  physicall and logical pins. 
//
#ifndef __SDK_GPIO_H
#define __SDK_GPIO_H

#ifdef __cplusplus
extern "C" {
#endif

//------------------------------------------------------------------------------
//
//  Define:  GPIO_DEVICE_NAME
//
#define GPIO_DEVICE_NAME        L"GIO1:"

//------------------------------------------------------------------------------
//
//  Define:  GPIO_DIR_xxx/GPIO_INT_xxx
//
#define GPIO_DIR_OUTPUT         (1 << 0)
#define GPIO_DIR_INPUT          (0 << 0)
#define GPIO_ODR_ENABLE         (1 << 0)
#define GPIO_ODR_DISABLE        (0 << 0)
#define GPIO_INT_EDGE_H_TO_L    (1 << 0)
#define GPIO_INT_EDGE_BOTH      (0 << 0)

//------------------------------------------------------------------------------
//
//  GUID:  DEVICE_IFC_GPIO_GUID
//
DEFINE_GUID(
    DEVICE_IFC_GPIO_GUID, 0xa0272611, 0xdea0, 0x4678,
    0xae, 0x62, 0x65, 0x61, 0x5b, 0x7d, 0x53, 0xaa
);

//------------------------------------------------------------------------------
// Enum : Gpio Numbers
//
typedef enum {
    GPIO_0 = 0,
    GPIO_1,
    GPIO_2,
    GPIO_3,
    GPIO_4,
    GPIO_5,
    GPIO_6,
    GPIO_7,
    GPIO_8,
    GPIO_9,
    GPIO_10,
    GPIO_11,
    GPIO_12,
    GPIO_13,
    GPIO_14,
    GPIO_15,
    GPIO_16,
    GPIO_17,
    GPIO_18,
    GPIO_19,
    GPIO_20,
    GPIO_21,
    GPIO_22,
    GPIO_23,
    GPIO_24,
    GPIO_25,
    GPIO_26,
    GPIO_27,
    GPIO_28,
    GPIO_29,
    GPIO_30,
    GPIO_31,

    //GPIOBank2
    GPIO_32,
    GPIO_33,
    GPIO_34,
    GPIO_35,
    GPIO_36,
    GPIO_37,
    GPIO_38,
    GPIO_39,
    GPIO_40,
    GPIO_41,
    GPIO_42,
    GPIO_43,
    GPIO_44,
    GPIO_45,
    GPIO_46,
    GPIO_47,
    GPIO_48,
    GPIO_49,
    GPIO_50,
    GPIO_51,
    GPIO_52,
    GPIO_53,
    GPIO_54,
    GPIO_55,
    GPIO_56,
    GPIO_57,
    GPIO_58,
    GPIO_59,
    GPIO_60,
    GPIO_61,
    GPIO_62,
    GPIO_63,

    //GPIOBank3
    GPIO_64,
    GPIO_65,
    GPIO_66,
    GPIO_67,
    GPIO_68,
    GPIO_69,
    GPIO_70,
    GPIO_71,
    GPIO_72,
    GPIO_73,
    GPIO_74,
    GPIO_75,
    GPIO_76,
    GPIO_77,
    GPIO_78,
    GPIO_79,
    GPIO_80,
    GPIO_81,
    GPIO_82,
    GPIO_83,
    GPIO_84,
    GPIO_85,
    GPIO_86,
    GPIO_87,
    GPIO_88,
    GPIO_89,
    GPIO_90,
    GPIO_91,
    GPIO_92,
    GPIO_93,
    GPIO_94,
    GPIO_95,

    //GPIOBank4
    GPIO_96,
    GPIO_97,
    GPIO_98,
    GPIO_99,
    GPIO_100,
    GPIO_101,
    GPIO_102,
    GPIO_103,
    GPIO_104,
    GPIO_105,
    GPIO_106,
    GPIO_107,
    GPIO_108,
    GPIO_109,
    GPIO_110,
    GPIO_111,
    GPIO_112,
    GPIO_113,
    GPIO_114,
    GPIO_115,
    GPIO_116,
    GPIO_117,
    GPIO_118,
    GPIO_119,
    GPIO_120,
    GPIO_121,
    GPIO_122,
    GPIO_123,
    GPIO_124,
    GPIO_125,
    GPIO_126,
    GPIO_127,

    GPIO_MAX_NUM
} GpioNum_e;

//------------------------------------------------------------------------------
//
//  Type:  DEVICE_CONTEXT_GPIO
//
//  This structure is used to store GPIO device context.
//
typedef struct {
    HANDLE hDevice;
} DEVICE_CONTEXT_GPIO;

//------------------------------------------------------------------------------

#define IOCTL_GPIO_WRITE_PIN       \
    CTL_CODE(FILE_DEVICE_UNKNOWN, 0x0300, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_GPIO_READ_PIN        \
    CTL_CODE(FILE_DEVICE_UNKNOWN, 0x0301, METHOD_BUFFERED, FILE_ANY_ACCESS)
    
#define IOCTL_GPIO_TOGGLE_PIN      \
    CTL_CODE(FILE_DEVICE_UNKNOWN, 0x0302, METHOD_BUFFERED, FILE_ANY_ACCESS)    

#define IOCTL_GPIO_SET_DIR         \
    CTL_CODE(FILE_DEVICE_UNKNOWN, 0x0303, METHOD_BUFFERED, FILE_ANY_ACCESS)    

#define IOCTL_GPIO_SET_ODR         \
    CTL_CODE(FILE_DEVICE_UNKNOWN, 0x0304, METHOD_BUFFERED, FILE_ANY_ACCESS)    

#define IOCTL_GPIO_SET_INT_TYPE    \
    CTL_CODE(FILE_DEVICE_UNKNOWN, 0x0305, METHOD_BUFFERED, FILE_ANY_ACCESS)    

#define IOCTL_GPIO_INIT_INTERRUPT  \
    CTL_CODE(FILE_DEVICE_UNKNOWN, 0x0306, METHOD_BUFFERED, FILE_ANY_ACCESS) 
    
#define IOCTL_GPIO_ACK_INTERRUPT   \
    CTL_CODE(FILE_DEVICE_UNKNOWN, 0x0307, METHOD_BUFFERED, FILE_ANY_ACCESS) 

#define IOCTL_GPIO_DISABLE_INTERRUPT   \
    CTL_CODE(FILE_DEVICE_UNKNOWN, 0x0308, METHOD_BUFFERED, FILE_ANY_ACCESS) 

#define IOCTL_GPIO_MASK_INTERRUPT   \
    CTL_CODE(FILE_DEVICE_UNKNOWN, 0x0309, METHOD_BUFFERED, FILE_ANY_ACCESS) 

#define IOCTL_MAP_PHYSIC_TO_UNCHACHED_VIRTUAL       \
    CTL_CODE(FILE_DEVICE_UNKNOWN, 0x030A, METHOD_BUFFERED, FILE_ANY_ACCESS) 

//------------------------------------------------------------------------------

typedef struct {
    UINT                    gpioId;
    UINT                    gpioState;
} IOCTL_GPIO_PIN;

typedef struct {
    UINT                    gpioId;
    UINT                    gpioDirODR;
} IOCTL_GPIO_SET_DIR_ODR;

typedef struct {
    UINT                    uSysIntr;
    HANDLE                  hEvent;
} IOCTL_GPIO_INIT_INTERRUPT_INFO,  *PIOCTL_GPIO_INIT_INTERRUPT_INFO;

typedef struct {
    UINT                    uSysIntr;
    BOOL                    bEnable;
} IOCTL_GPIO_INTERRUPT_MASK;

typedef struct {
    DWORD       idDestProc;
    void*       pvDestMem;
    UINT        physAddr;
    UINT        size;
} VIRTUAL_COPY_EX_DATA;

//------------------------------------------------------------------------------

__inline HANDLE
GPIOOpen(
    )
{
    HANDLE hDevice = INVALID_HANDLE_VALUE;
    DEVICE_CONTEXT_GPIO *pContext = NULL;

    hDevice = CreateFile(GPIO_DEVICE_NAME, 0, 0, NULL, 0, 0, NULL);
    if (hDevice == INVALID_HANDLE_VALUE) goto cleanUp;

    // Allocate memory for our handler...
    pContext = (DEVICE_CONTEXT_GPIO*)LocalAlloc(
        LPTR, sizeof(DEVICE_CONTEXT_GPIO)
        );
    if (pContext == NULL)
        {
        CloseHandle(hDevice);
        goto cleanUp;
        }

    // Save device handle
    pContext->hDevice = hDevice;

cleanUp:
    return pContext;
}

__inline VOID
GPIOClose(
    HANDLE hContext
    )
{
    DEVICE_CONTEXT_GPIO *pContext = (DEVICE_CONTEXT_GPIO *)hContext;
    if (pContext != NULL)
        {
        CloseHandle(pContext->hDevice);
        LocalFree(pContext);
        }
}

__inline VOID
GPIOWritePin(
    HANDLE hContext,
    DWORD  pin,
    DWORD  state
    )
{
    DEVICE_CONTEXT_GPIO *pContext = (DEVICE_CONTEXT_GPIO *)hContext;
    IOCTL_GPIO_PIN       ioGPIOpin;

    if( pContext == NULL )
        {
        return;
        }

    ioGPIOpin.gpioId        = pin;
    ioGPIOpin.gpioState     = state;
    DeviceIoControl(pContext->hDevice,
                        IOCTL_GPIO_WRITE_PIN,
                        &ioGPIOpin,
                        sizeof(ioGPIOpin),
                        NULL,
                        0,
                        NULL,
                        NULL );
}

__inline DWORD
GPIOReadPin(
    HANDLE hContext,
    DWORD  pin
    )
{
    DEVICE_CONTEXT_GPIO *pContext = (DEVICE_CONTEXT_GPIO *)hContext;
    DWORD   dwValue;

    if( pContext == NULL )
        {
        return 0;
        }


    dwValue = 0;

    DeviceIoControl(pContext->hDevice,
                    IOCTL_GPIO_READ_PIN,
                    &pin,
                    sizeof(pin),
                    &dwValue,
                    sizeof(dwValue),
                    NULL,
                    NULL );

    return dwValue;
}

__inline VOID
GPIOTogglePin(
    HANDLE hContext,
    DWORD  pin
    )
{
    DEVICE_CONTEXT_GPIO *pContext = (DEVICE_CONTEXT_GPIO *)hContext;

    if( pContext == NULL )
        {
        return;
        }

    DeviceIoControl(pContext->hDevice,
                    IOCTL_GPIO_TOGGLE_PIN,
                    &pin,
                    sizeof(pin),
                    NULL,
                    0,
                    NULL,
                    NULL );
}

__inline VOID
GPIOSetDir(
    HANDLE hContext,
    DWORD  pin,
    DWORD  dir
    )
{
    DEVICE_CONTEXT_GPIO *pContext = (DEVICE_CONTEXT_GPIO *)hContext;
    IOCTL_GPIO_SET_DIR_ODR  ioGPIOdodr;

    if( pContext == NULL )
        {
        return;
        }

    ioGPIOdodr.gpioId     = pin;
    ioGPIOdodr.gpioDirODR = dir;
    DeviceIoControl(pContext->hDevice,
                    IOCTL_GPIO_SET_DIR,
                    &ioGPIOdodr,
                    sizeof(ioGPIOdodr),
                    NULL,
                    0,
                    NULL,
                    NULL );
}

__inline VOID
GPIOSetOpenDrain(
    HANDLE hContext,
    DWORD  pin,
    DWORD  state
    )
{
    DEVICE_CONTEXT_GPIO *pContext = (DEVICE_CONTEXT_GPIO *)hContext;
    IOCTL_GPIO_SET_DIR_ODR  ioGPIOdodr;

    if( pContext == NULL )
        {
        return;
        }

    ioGPIOdodr.gpioId     = pin;
    ioGPIOdodr.gpioDirODR = state;
    DeviceIoControl(pContext->hDevice,
                    IOCTL_GPIO_SET_ODR,
                    &ioGPIOdodr,
                    sizeof(ioGPIOdodr),
                    NULL,
                    0,
                    NULL,
                    NULL );
}

__inline VOID
GPIOSetInterruptType(
    HANDLE hContext,
    DWORD  pin,
    DWORD  type
    )
{
    DEVICE_CONTEXT_GPIO *pContext = (DEVICE_CONTEXT_GPIO *)hContext;
    IOCTL_GPIO_SET_DIR_ODR  ioGPIOdodr;

    if( pContext == NULL )
        {
        return;
        }

    ioGPIOdodr.gpioId     = pin;
    ioGPIOdodr.gpioDirODR = type;
    DeviceIoControl(pContext->hDevice,
                    IOCTL_GPIO_SET_INT_TYPE,
                    &ioGPIOdodr,
                    sizeof(ioGPIOdodr),
                    NULL,
                    0,
                    NULL,
                    NULL );
}

__inline VOID
GPIOInterruptInitialize(
    HANDLE hContext,
    DWORD  dwSysISR,
    HANDLE hEvent
    )
{
    DEVICE_CONTEXT_GPIO *pContext = (DEVICE_CONTEXT_GPIO *)hContext;
    IOCTL_GPIO_INIT_INTERRUPT_INFO  igiii;

    if( pContext == NULL )
        {
        return;
        }

    igiii.uSysIntr     = dwSysISR;
    igiii.hEvent       = hEvent;
    DeviceIoControl(pContext->hDevice,
                    IOCTL_GPIO_INIT_INTERRUPT,
                    &igiii,
                    sizeof(igiii),
                    NULL,
                    0,
                    NULL,
                    NULL );
}

__inline VOID
GPIOInterruptDone(
    HANDLE hContext,
    DWORD  dwSysISR
    )
{
    DEVICE_CONTEXT_GPIO *pContext = (DEVICE_CONTEXT_GPIO *)hContext;

    if( pContext == NULL )
        {
        return;
        }

    DeviceIoControl(pContext->hDevice,
                    IOCTL_GPIO_ACK_INTERRUPT,
                    &dwSysISR,
                    sizeof(dwSysISR),
                    NULL,
                    0,
                    NULL,
                    NULL );
}

__inline VOID
GPIOInterruptDisable(
    HANDLE hContext,
    DWORD  dwSysISR
    )
{
    DEVICE_CONTEXT_GPIO *pContext = (DEVICE_CONTEXT_GPIO *)hContext;

    if( pContext == NULL )
        {
        return;
        }

    DeviceIoControl(pContext->hDevice,
                    IOCTL_GPIO_DISABLE_INTERRUPT,
                    &dwSysISR,
                    sizeof(dwSysISR),
                    NULL,
                    0,
                    NULL,
                    NULL );
}

__inline VOID
GPIOInterruptMask(
    HANDLE hContext,
    DWORD  dwSysISR,
    BOOL   bEnable
    )
{
    DEVICE_CONTEXT_GPIO *pContext = (DEVICE_CONTEXT_GPIO *)hContext;

    if( pContext == NULL )
        {
        return;
        }

    DeviceIoControl(pContext->hDevice,
                    IOCTL_GPIO_MASK_INTERRUPT,
                    &dwSysISR,
                    sizeof(dwSysISR),
                    NULL,
                    0,
                    NULL,
                    NULL );
}

__inline VOID *
GPIO_MmMapIoSpace(
    HANDLE hContext,
    DWORD  dwPhysAddr,
    DWORD  dwWindowSize
    )
{
    DEVICE_CONTEXT_GPIO *   pContext = (DEVICE_CONTEXT_GPIO *)hContext;
    VIRTUAL_COPY_EX_DATA    vced;
    volatile LPVOID         pVirtual;


    if( pContext == NULL )
        {
        return NULL;
        }

    vced.idDestProc     = GetCurrentProcessId();
    vced.physAddr       = dwPhysAddr;
    vced.size           = dwWindowSize;
    
    pVirtual = VirtualAlloc (0, dwWindowSize, MEM_RESERVE, PAGE_NOACCESS);
    
    if (pVirtual == NULL)
        return NULL;
        
    vced.pvDestMem = pVirtual;
    
    if (!DeviceIoControl(pContext->hDevice,
                    IOCTL_MAP_PHYSIC_TO_UNCHACHED_VIRTUAL,
                    &vced,
                    sizeof(vced),
                    NULL,
                    0,
                    NULL,
                    NULL ))
       {
            VirtualFree(pVirtual, 0, MEM_RELEASE);
            return NULL;
       }
                    
    return pVirtual;
}


__inline DWORD
GPIOIoControl(
    HANDLE hContext,
    DWORD  code,
    UCHAR *pInBuffer,
    DWORD  inSize,
    UCHAR *pOutBuffer,
    DWORD  outSize,
    DWORD *pOutSize,
    LPOVERLAPPED pOverlap
    )
{
    DEVICE_CONTEXT_GPIO *pContext = (DEVICE_CONTEXT_GPIO *)hContext;

    if ( pContext == NULL )
        {
        return 0;
        }

    return DeviceIoControl(pContext->hDevice, code, pInBuffer,
                inSize, pOutBuffer, outSize, pOutSize, pOverlap);
}

//------------------------------------------------------------------------------

#ifdef __cplusplus
}
#endif

#endif
 